!*****************************************************************!
!*                                                               *!
!*              4C (FORESEE) Simulation Model                    *!
!*                                                               *!
!*                    Subroutines for:                           *!
!*    - dimsort: sorting of cohorts according to a               *!
!*       charcteristic variable                                  *! 
!*    - sort2: subroutine from Numerical recipes                 *!
!*                                                               *!
!*                  Copyright (C) 1996-2018                      *!
!*     Potsdam Institute for Climate Impact Reserach (PIK)       *!
!*          Authors and contributors see AUTHOR file             *!
!*  This file is part of 4C and is licensed under BSD-2-Clause   *!
!*                   See LICENSE file or under:                  *!
!*     http://www.https://opensource.org/licenses/BSD-2-Clause   *!
!*                           Contact:                            *!
!*       https://gitlab.pik-potsdam.de/foresee/4C                *!
!*                                                               *!
!*****************************************************************!

SUBROUTINE dimsort(n,var,ranktable)

  USE data_species
  USE data_stand           ! state variables of stand, cohort and cohort element   
  IMPLICIT NONE
  INTEGER :: isort,n
  INTEGER :: ranktable(n)
  CHARACTER(3) :: var
  REAL    :: sortarray(n)
 
    ranktable=0
    sortarray=0
    isort=1
    zeig=>pt%first
    DO 
      IF (.not.ASSOCIATED(zeig)) exit
	  IF (zeig%coh%species .le. nspec_tree) THEN    ! for trees only
         ranktable(isort) = zeig%coh%ident
         IF(var=='hei') sortarray(isort) = zeig%coh%height
         IF(var=='dbh') sortarray(isort) = zeig%coh%diam
         isort=isort+1
      ENDIF
	  zeig=>zeig%next
    END DO 
    CALL sort2(n,sortarray,ranktable)
END SUBROUTINE dimsort

!******************************************************************************

      SUBROUTINE sort2(n,arr,brr)
! sorts array arr(1:n) into an ascending order and 
! makes the corresponding rearrangement of the array brr(1:n)      

      INTEGER n,M,NSTACK

      REAL arr(n)
      INTEGER brr(n)

      PARAMETER (M=7,NSTACK=50)

      INTEGER i,ir,j,jstack,k,l,istack(NSTACK)

      REAL a,b,temp
      jstack=0
      l=1
      ir=n
1     if(ir-l.lt.M)then
        do 12 j=l+1,ir
          a=arr(j)
          b=brr(j)
          do 11 i=j-1,1,-1
            if(arr(i).le.a)goto 2
            arr(i+1)=arr(i)
            brr(i+1)=brr(i)
11        continue
          i=0
2         arr(i+1)=a
          brr(i+1)=b
12      continue
        if(jstack.eq.0)return
        ir=istack(jstack)
        l=istack(jstack-1)
        jstack=jstack-2
      else
        k=(l+ir)/2
        temp=arr(k)
        arr(k)=arr(l+1)
        arr(l+1)=temp
        temp=brr(k)
        brr(k)=brr(l+1)
        brr(l+1)=temp
        if(arr(l+1).gt.arr(ir))then
          temp=arr(l+1)
          arr(l+1)=arr(ir)
          arr(ir)=temp
          temp=brr(l+1)
          brr(l+1)=brr(ir)
          brr(ir)=temp
        endif
        if(arr(l).gt.arr(ir))then
          temp=arr(l)
          arr(l)=arr(ir)
          arr(ir)=temp
          temp=brr(l)
          brr(l)=brr(ir)
          brr(ir)=temp
        endif

        if(arr(l+1).gt.arr(l))then
         temp=arr(l+1)
          arr(l+1)=arr(l)
          arr(l)=temp
          temp=brr(l+1)
          brr(l+1)=brr(l)
          brr(l)=temp
        endif

        i=l+1
        j=ir
        a=arr(l)
        b=brr(l)
3       continue

          i=i+1
        if(arr(i).lt.a)goto 3
4       continue

          j=j-1
        if(arr(j).gt.a)goto 4
        if(j.lt.i)goto 5
        temp=arr(i)
        arr(i)=arr(j)
        arr(j)=temp
        temp=brr(i)
        brr(i)=brr(j)
        brr(j)=temp
        goto 3

5       arr(l)=arr(j)
        arr(j)=a
        brr(l)=brr(j)
        brr(j)=b
        jstack=jstack+2
        if(jstack.gt.NSTACK)pause 'NSTACK too small in sort2'
        if(ir-i+1.ge.j-l)then
          istack(jstack)=ir
          istack(jstack-1)=i
          ir=j-1
        else

          istack(jstack)=j-1
          istack(jstack-1)=l
          l=i
        endif

      endif

      goto 1

      END Subroutine

!  (C) Copr. 1986-92 Numerical Recipes Software "!D#+.

